@@ -4,17 +4,16 @@ from streamlink.plugin import Plugin
 from streamlink.plugin.api import validate
 from streamlink.stream import HLSStream
 from streamlink import NoStreamsError
-from streamlink.utils import parse_json
+
 
 class CubeTV(Plugin):
-
     _url_re = re.compile(r"https?://(www\.)?cube\.tv/(?P<channel>[^/]{2,})")
 
     _channel_info_api_url_base = "https://www.cube.tv/studio/info?cube_id={channel}"
     _stream_data_api_url_base = "https://www.cube.tv/studioApi/getStudioSrcBySid?sid={gid}&videoType=1&https=1"
 
     _channel_info_schema = validate.Schema({
-        u"code" : 1,
+        u"code": 1,
         u"msg": u"success",
         u"data": {
             u"gid": validate.text,
@@ -23,7 +22,7 @@ class CubeTV(Plugin):
     })
 
     _stream_data_schema = validate.Schema({
-        u"code" : 1,
+        u"code": 1,
         u"msg": u"success",
         u"data": {
             u"video": u"hls",
@@ -39,7 +38,7 @@ class CubeTV(Plugin):
         try:
             res = self.session.http.get(self._channel_info_api_url_base.format(channel=user_id))
             return res
-        except Exception as e:
+        except Exception:
             raise NoStreamsError(self.url)
 
     def _get_streams(self):
@@ -50,7 +49,7 @@ class CubeTV(Plugin):
         try:
             stream_data = self.session.http.get(self._stream_data_api_url_base.format(gid=user_gid))
             hls = self.session.http.json(stream_data, schema=self._stream_data_schema)['data']['video_src']
-        except Exception as e:
+        except Exception:
             raise NoStreamsError(self.url)
 
         return HLSStream.parse_variant_playlist(self.session, hls)
