@@ -8,6 +8,8 @@ import time
 
 from collections import defaultdict, namedtuple
 from itertools import repeat, count
+
+import math
 from isodate import parse_datetime, parse_duration, Duration
 from contextlib import contextmanager
 from streamlink.compat import urlparse, urljoin, urlunparse, izip, urlsplit, urlunsplit
@@ -32,12 +34,13 @@ epoch_start = datetime.datetime(1970, 1, 1, tzinfo=utc)
 
 
 class Segment(object):
-    def __init__(self, url, duration, init=False, content=True, available_at=epoch_start):
+    def __init__(self, url, duration, init=False, content=True, available_at=epoch_start, range=None):
         self.url = url
         self.duration = duration
         self.init = init
         self.content = content
         self.available_at = available_at
+        self.range = range
 
 
 def datetime_to_seconds(dt):
@@ -119,6 +122,15 @@ class MPDParsers(object):
 
         return _timedelta
 
+    @staticmethod
+    def range(range_spec):
+        r = range_spec.split("-")
+        if len(r) != 2:
+            raise MPDParsingError("invalid byte-range-spec")
+
+        start, end = int(r[0]), r[1] and int(r[1]) or None
+        return start, end and ((end - start) + 1)
+
 
 class MPDParsingError(Exception):
     pass
@@ -322,9 +334,52 @@ class EventStream(MPDNode):
     __tag__ = "EventStream"
 
 
+class Initialization(MPDNode):
+    __tag__ = "Initialization"
+
+    def __init__(self, node, root=None, parent=None, *args, **kwargs):
+        super(Initialization, self).__init__(node, root, parent, *args, **kwargs)
+        self.source_url = self.attr("sourceURL")
+
+
+class SegmentURL(MPDNode):
+    __tag__ = "SegmentURL"
+
+    def __init__(self, node, root=None, parent=None, *args, **kwargs):
+        super(SegmentURL, self).__init__(node, root, parent, *args, **kwargs)
+        self.media = self.attr("media")
+        self.media_range = self.attr("mediaRange", parser=MPDParsers.range)
+
+
 class SegmentList(MPDNode):
     __tag__ = "SegmentList"
 
+    def __init__(self, node, root=None, parent=None, *args, **kwargs):
+        super(SegmentList, self).__init__(node, root, parent, *args, **kwargs)
+
+        self.presentation_time_offset = self.attr("presentationTimeOffset")
+        self.timescale = self.attr("timescale", parser=int)
+        self.duration = self.attr("duration", parser=int)
+        self.start_number = self.attr("startNumber", parser=int, default=1)
+
+        if self.duration:
+            self.duration_seconds = self.duration / float(self.timescale)
+        else:
+            self.duration_seconds = None
+
+        self.initialization = self.only_child(Initialization)
+        self.segment_urls = self.children(SegmentURL, minimum=1)
+
+    @property
+    def segments(self):
+        if self.initialization:
+            yield Segment(self.make_url(self.initialization.source_url), 0, init=True, content=False)
+        for n, segment_url in enumerate(self.segment_urls, self.start_number):
+            yield Segment(self.make_url(segment_url.media), self.duration_seconds, range=segment_url.media_range)
+
+    def make_url(self, url):
+        return BaseURL.join(self.base_url, url)
+
 
 class AdaptationSet(MPDNode):
     __tag__ = u"AdaptationSet"
@@ -440,9 +495,13 @@ class SegmentTemplate(MPDNode):
                                                           else 3))
 
             # the number of the segment that is available at NOW - SUGGESTED_DELAY - BUFFER_TIME
-            number_iter = count(self.startNumber +
-                                int((since_start - suggested_delay - self.root.minBufferTime).total_seconds() /
-                                    self.duration_seconds))
+            number_iter = count(
+                self.startNumber
+                + int(
+                    (since_start - suggested_delay - self.root.minBufferTime).total_seconds()
+                    / self.duration_seconds
+                )
+            )
 
             # the time the segment number is available at NOW
             available_iter = count_dt(available_start,
@@ -453,6 +512,10 @@ class SegmentTemplate(MPDNode):
 
     def format_media(self, **kwargs):
         if self.segmentTimeline:
+            if self.parent.id is None:
+                # workaround for invalid `self.root.timelines[self.parent.id]`
+                # creates a timeline for every mimeType instead of one for both
+                self.parent.id = self.parent.mimeType
             log.debug("Generating segment timeline for {0} playlist (id={1}))".format(self.root.type, self.parent.id))
             if self.root.type == "dynamic":
                 # if there is no delay, use a delay of 3 seconds
@@ -488,7 +551,7 @@ class SegmentTemplate(MPDNode):
             else:
                 for segment, n in zip(self.segmentTimeline.segments, count(self.startNumber)):
                     yield (self.make_url(self.media(Time=segment.t, Number=n, **kwargs)),
-                           datetime.timedelta(seconds=segment.d / self.timescale))
+                           datetime.datetime.now(tz=utc))
 
         else:
             for number, available_at in self.segment_numbers():
@@ -523,9 +586,13 @@ class Representation(MPDNode):
         self.baseURLs = self.children(BaseURL)
         self.subRepresentation = self.children(SubRepresentation)
         self.segmentBase = self.only_child(SegmentBase)
-        self.segmentList = self.only_child(SegmentList)
+        self.segmentList = self.children(SegmentList)
         self.segmentTemplate = self.only_child(SegmentTemplate)
 
+    @property
+    def bandwidth_rounded(self):
+        return round(self.bandwidth, 1 - int(math.log10(self.bandwidth)))
+
     def segments(self, **kwargs):
         """
         Segments are yielded when they are available
@@ -537,8 +604,8 @@ class Representation(MPDNode):
         :return: yields Segments
         """
 
-        segmentBase = self.segmentBase or self.walk_back_get_attr("segmentBase")
-        segmentList = self.segmentList or self.walk_back_get_attr("segmentList")
+        # segmentBase = self.segmentBase or self.walk_back_get_attr("segmentBase")
+        segmentLists = self.segmentList or self.walk_back_get_attr("segmentList")
         segmentTemplate = self.segmentTemplate or self.walk_back_get_attr("segmentTemplate")
 
         if segmentTemplate:
@@ -549,6 +616,10 @@ class Representation(MPDNode):
                     yield segment
                 else:
                     yield segment
+        elif segmentLists:
+            for segmentList in segmentLists:
+                for segment in segmentList.segments:
+                    yield segment
         else:
             yield Segment(self.base_url, 0, True, True)
 
@@ -572,7 +643,8 @@ class SegmentTimeline(MPDNode):
     def segments(self):
         t = 0
         for tsegment in self.timeline_segments:
-            t = t or tsegment.t
+            if t == 0 and tsegment.t is not None:
+                t = tsegment.t
             # check the start time from MPD
             for repeated_i in range(tsegment.r + 1):
                 yield self.TimelineSegment(t, tsegment.d)
