@@ -5,7 +5,7 @@ from functools import partial
 from streamlink.plugins.twitch import Twitch, TwitchHLSStream
 
 import requests_mock
-from tests.mock import call, patch
+from tests.mock import MagicMock, call, patch
 
 from streamlink.session import Streamlink
 from tests.resources import text
@@ -34,17 +34,21 @@ class TestPluginTwitch(unittest.TestCase):
             self.assertFalse(Twitch.can_handle_url(url))
 
 
+@patch("streamlink.stream.hls.HLSStreamWorker.wait", MagicMock(return_value=True))
 class TestTwitchHLSStream(unittest.TestCase):
-    scte35_out = "#EXT-X-DISCONTINUITY\n#EXT-X-SCTE35-OUT\n"
-    scte35_out_cont = "#EXT-X-SCTE35-OUT-CONT\n"
-    scte35_in = "#EXT-X-DISCONTINUITY\n#EXT-X-SCTE35-IN\n"
+    url_master = "http://mocked/path/master.m3u8"
+    url_playlist = "http://mocked/path/playlist.m3u8"
+    url_segment = "http://mocked/path/stream{0}.ts"
+
     segment = "#EXTINF:1.000,\nstream{0}.ts\n"
+    segment_ad = "#EXTINF:1.000,Amazon|123456789\nstream{0}.ts\n"
+    prefetch = "#EXT-X-TWITCH-PREFETCH:{0}\n"
 
     def getMasterPlaylist(self):
         with text("hls/test_master.m3u8") as pl:
             return pl.read()
 
-    def getPlaylist(self, media_sequence, items):
+    def getPlaylist(self, media_sequence, items, ads=False, prefetch=None):
         playlist = """
 #EXTM3U
 #EXT-X-VERSION:5
@@ -52,140 +56,150 @@ class TestTwitchHLSStream(unittest.TestCase):
 #EXT-X-MEDIA-SEQUENCE:{0}
 """.format(media_sequence)
 
+        segment = self.segment if not ads else self.segment_ad
         for item in items:
-            if type(item) != int:
-                playlist += item
-            else:
-                playlist += self.segment.format(item)
+            playlist += segment.format(item)
+        for item in prefetch or []:
+            playlist += self.prefetch.format(self.url_segment.format(item))
 
         return playlist
 
-    def start_streamlink(self, kwargs=None):
+    def start_streamlink(self, disable_ads=False, low_latency=False, kwargs=None):
         kwargs = kwargs or {}
         log.info("Executing streamlink")
         streamlink = Streamlink()
 
         streamlink.set_option("hls-live-edge", 4)
-        streamlink.plugins.get("twitch").options.set("disable-ads", True)
+        streamlink.set_plugin_option("twitch", "disable-ads", disable_ads)
+        streamlink.set_plugin_option("twitch", "low-latency", low_latency)
 
-        masterStream = TwitchHLSStream.parse_variant_playlist(
-            streamlink,
-            "http://mocked/path/master.m3u8",
-            **kwargs
-        )
+        masterStream = TwitchHLSStream.parse_variant_playlist(streamlink, self.url_master, **kwargs)
         stream = masterStream["1080p (source)"].open()
         data = b"".join(iter(partial(stream.read, 8192), b""))
         stream.close()
         log.info("End of streamlink execution")
-        return data
+        return streamlink, data
 
     def mock(self, mocked, method, url, *args, **kwargs):
         mocked[url] = method(url, *args, **kwargs)
 
-    def get_result(self, streams, playlists):
+    def get_result(self, streams, playlists, **kwargs):
         mocked = {}
         with requests_mock.Mocker() as mock:
-            self.mock(mocked, mock.get, "http://mocked/path/master.m3u8", text=self.getMasterPlaylist())
-            self.mock(mocked, mock.get, "http://mocked/path/playlist.m3u8", [{"text": p} for p in playlists])
+            self.mock(mocked, mock.get, self.url_master, text=self.getMasterPlaylist())
+            self.mock(mocked, mock.get, self.url_playlist, [{"text": p} for p in playlists])
             for i, stream in enumerate(streams):
-                self.mock(mocked, mock.get, "http://mocked/path/stream{0}.ts".format(i), content=stream)
-            return self.start_streamlink(), mocked
+                self.mock(mocked, mock.get, self.url_segment.format(i), content=stream)
+            streamlink, data = self.start_streamlink(**kwargs)
+            return streamlink, data, mocked
 
     @patch("streamlink.plugins.twitch.log")
-    def test_hls_scte35_start_with_end(self, mock_logging):
-        streams = ["[{0}]".format(i).encode("ascii") for i in range(12)]
+    def test_hls_disable_ads_preroll(self, mock_logging):
+        streams = ["[{0}]".format(i).encode("ascii") for i in range(6)]
         playlists = [
-            self.getPlaylist(0, [self.scte35_out, 0, 1, 2, 3]),
-            self.getPlaylist(4, [self.scte35_in, 4, 5, 6, 7]),
-            self.getPlaylist(8, [8, 9, 10, 11]) + "#EXT-X-ENDLIST\n"
+            self.getPlaylist(0, [0, 1], ads=True),
+            self.getPlaylist(2, [2, 3], ads=True),
+            self.getPlaylist(4, [4, 5]) + "#EXT-X-ENDLIST\n"
         ]
-        result, mocked = self.get_result(streams, playlists)
+        streamlink, result, mocked = self.get_result(streams, playlists, disable_ads=True)
 
-        expected = b''.join(streams[4:12])
-        self.assertEqual(expected, result)
-        for i, _ in enumerate(streams):
-            self.assertTrue(mocked["http://mocked/path/stream{0}.ts".format(i)].called)
+        self.assertEqual(result, b''.join(streams[4:6]))
+        for i in range(0, 6):
+            self.assertTrue(mocked[self.url_segment.format(i)].called, i)
         mock_logging.info.assert_has_calls([
             call("Will skip ad segments"),
-            call("Will skip ads beginning with segment 0"),
-            call("Will stop skipping ads beginning with segment 4")
+            call("Waiting for pre-roll ads to finish, be patient")
         ])
+        self.assertEqual(mock_logging.info.call_count, 2)
 
     @patch("streamlink.plugins.twitch.log")
-    def test_hls_scte35_no_start(self, mock_logging):
-        streams = ["[{0}]".format(i).encode("ascii") for i in range(8)]
+    def test_hls_disable_ads_no_preroll(self, mock_logging):
+        streams = ["[{0}]".format(i).encode("ascii") for i in range(6)]
         playlists = [
-            self.getPlaylist(0, [0, 1, 2, 3]),
-            self.getPlaylist(4, [self.scte35_in, 4, 5, 6, 7]) + "#EXT-X-ENDLIST\n"
+            self.getPlaylist(0, [0, 1]),
+            self.getPlaylist(2, [2, 3], ads=True),
+            self.getPlaylist(4, [4, 5]) + "#EXT-X-ENDLIST\n"
         ]
-        result, mocked = self.get_result(streams, playlists)
+        streamlink, result, mocked = self.get_result(streams, playlists, disable_ads=True)
 
-        expected = b''.join(streams[0:8])
-        self.assertEqual(expected, result)
-        for i, _ in enumerate(streams):
-            self.assertTrue(mocked["http://mocked/path/stream{0}.ts".format(i)].called)
+        self.assertEqual(result, b''.join(streams[0:2]) + b''.join(streams[4:6]))
+        for i in range(0, 6):
+            self.assertTrue(mocked[self.url_segment.format(i)].called, i)
         mock_logging.info.assert_has_calls([
             call("Will skip ad segments")
         ])
 
     @patch("streamlink.plugins.twitch.log")
-    def test_hls_scte35_no_start_with_cont(self, mock_logging):
-        streams = ["[{0}]".format(i).encode("ascii") for i in range(8)]
+    def test_hls_no_disable_ads(self, mock_logging):
+        streams = ["[{0}]".format(i).encode("ascii") for i in range(4)]
         playlists = [
-            self.getPlaylist(0, [self.scte35_out_cont, 0, 1, 2, 3]),
-            self.getPlaylist(4, [self.scte35_in, 4, 5, 6, 7]) + "#EXT-X-ENDLIST\n"
+            self.getPlaylist(0, [0, 1], ads=True),
+            self.getPlaylist(2, [2, 3]) + "#EXT-X-ENDLIST\n"
         ]
-        result, mocked = self.get_result(streams, playlists)
+        streamlink, result, mocked = self.get_result(streams, playlists, disable_ads=False)
 
-        expected = b''.join(streams[4:8])
+        self.assertEqual(result, b''.join(streams[0:4]))
+        for i in range(0, 4):
+            self.assertTrue(mocked[self.url_segment.format(i)].called, i)
+        mock_logging.info.assert_has_calls([])
+
+    @patch("streamlink.plugins.twitch.log")
+    def test_hls_prefetch(self, mock_logging):
+        streams = ["[{0}]".format(i).encode("ascii") for i in range(10)]
+        playlists = [
+            self.getPlaylist(0, [0, 1, 2, 3], prefetch=[4, 5]),
+            self.getPlaylist(4, [4, 5, 6, 7], prefetch=[8, 9]) + "#EXT-X-ENDLIST\n"
+        ]
+        streamlink, result, mocked = self.get_result(streams, playlists, low_latency=True)
+
+        self.assertEqual(2, streamlink.options.get("hls-live-edge"))
+        self.assertEqual(True, streamlink.options.get("hls-segment-stream-data"))
+
+        expected = b''.join(streams[4:10])
         self.assertEqual(expected, result)
-        for i, _ in enumerate(streams):
-            self.assertTrue(mocked["http://mocked/path/stream{0}.ts".format(i)].called)
+        for i in range(0, 3):
+            self.assertFalse(mocked[self.url_segment.format(i)].called, i)
+        for i in range(4, 9):
+            self.assertTrue(mocked[self.url_segment.format(i)].called, i)
         mock_logging.info.assert_has_calls([
-            call("Will skip ad segments"),
-            call("Will skip ads beginning with segment 0"),
-            call("Will stop skipping ads beginning with segment 4")
+            call("Low latency streaming (HLS live edge: 2)")
         ])
 
     @patch("streamlink.plugins.twitch.log")
-    def test_hls_scte35_no_end(self, mock_logging):
-        streams = ["[{0}]".format(i).encode("ascii") for i in range(12)]
+    def test_hls_prefetch_no_low_latency(self, mock_logging):
+        streams = ["[{0}]".format(i).encode("ascii") for i in range(10)]
         playlists = [
-            self.getPlaylist(0, [0, 1, 2, 3]),
-            self.getPlaylist(4, [self.scte35_out, 4, 5, 6, 7]),
-            self.getPlaylist(8, [8, 9, 10, 11]) + "#EXT-X-ENDLIST\n"
+            self.getPlaylist(0, [0, 1, 2, 3], prefetch=[4, 5]),
+            self.getPlaylist(4, [4, 5, 6, 7], prefetch=[8, 9]) + "#EXT-X-ENDLIST\n"
         ]
-        result, mocked = self.get_result(streams, playlists)
+        streamlink, result, mocked = self.get_result(streams, playlists)
 
-        expected = b''.join(streams[0:4])
+        self.assertEqual(4, streamlink.options.get("hls-live-edge"))
+        self.assertEqual(False, streamlink.options.get("hls-segment-stream-data"))
+
+        expected = b''.join(streams[0:8])
         self.assertEqual(expected, result)
-        for i, _ in enumerate(streams):
-            self.assertTrue(mocked["http://mocked/path/stream{0}.ts".format(i)].called)
-        mock_logging.info.assert_has_calls([
-            call("Will skip ad segments"),
-            call("Will skip ads beginning with segment 4")
-        ])
+        for i in range(0, 7):
+            self.assertTrue(mocked[self.url_segment.format(i)].called, i)
+        for i in range(8, 9):
+            self.assertFalse(mocked[self.url_segment.format(i)].called, i)
+        mock_logging.info.assert_has_calls([])
 
     @patch("streamlink.plugins.twitch.log")
-    def test_hls_scte35_in_between(self, mock_logging):
-        streams = ["[{0}]".format(i).encode("ascii") for i in range(20)]
+    def test_hls_no_low_latency_no_prefetch(self, mock_logging):
+        streams = ["[{0}]".format(i).encode("ascii") for i in range(10)]
         playlists = [
-            self.getPlaylist(0, [0, 1, 2, 3]),
-            self.getPlaylist(4, [4, 5, self.scte35_out, 6, 7]),
-            self.getPlaylist(8, [8, 9, 10, 11]),
-            self.getPlaylist(12, [12, 13, self.scte35_in, 14, 15]),
-            self.getPlaylist(16, [16, 17, 18, 19]) + "#EXT-X-ENDLIST\n"
+            self.getPlaylist(0, [0, 1, 2, 3], prefetch=[]),
+            self.getPlaylist(4, [4, 5, 6, 7], prefetch=[]) + "#EXT-X-ENDLIST\n"
         ]
-        result, mocked = self.get_result(streams, playlists)
+        streamlink, result, mocked = self.get_result(streams, playlists, low_latency=True)
+
+        self.assertTrue(streamlink.get_plugin_option("twitch", "low-latency"))
+        self.assertFalse(streamlink.get_plugin_option("twitch", "disable-ads"))
 
-        expected = b''.join(streams[0:6]) + b''.join(streams[14:20])
-        self.assertEqual(expected, result)
-        for i, _ in enumerate(streams):
-            self.assertTrue(mocked["http://mocked/path/stream{0}.ts".format(i)].called)
         mock_logging.info.assert_has_calls([
-            call("Will skip ad segments"),
-            call("Will skip ads beginning with segment 6"),
-            call("Will stop skipping ads beginning with segment 14")
+            call("Low latency streaming (HLS live edge: 2)"),
+            call("This is not a low latency stream")
         ])
 
 
@@ -196,7 +210,6 @@ class TestTwitchReruns(unittest.TestCase):
     class StopError(Exception):
         """Stop when trying to get an access token in _get_hls_streams..."""
 
-    @patch("streamlink.plugins.twitch.Twitch._authenticate", return_value=None)
     @patch("streamlink.plugins.twitch.Twitch._check_for_host", return_value=None)
     @patch("streamlink.plugins.twitch.Twitch._access_token", side_effect=StopError())
     def start(self, *mocked, **params):
@@ -208,8 +221,11 @@ class TestTwitchReruns(unittest.TestCase):
             mocked_stream = mock.get(
                 "https://api.twitch.tv/kraken/streams/1234.json",
                 json={"stream": None} if params.pop("offline", False) else {"stream": {
-                    "stream_type": params.pop("stream_type", "rerun"),
-                    "broadcast_platform": params.pop("broadcast_platform", "rerun")
+                    "stream_type": params.pop("stream_type", "live"),
+                    "broadcast_platform": params.pop("broadcast_platform", "live"),
+                    "channel": {
+                        "broadcaster_software": params.pop("broadcaster_software", "")
+                    }
                 }}
             )
 
@@ -220,13 +236,21 @@ class TestTwitchReruns(unittest.TestCase):
             try:
                 streams = plugin.streams()
             except TestTwitchReruns.StopError:
-                streams = None
+                streams = True
                 pass
 
             return streams, mocked_users, mocked_stream, mocked[0]
 
-    def test_disable_reruns(self, mock_log):
-        streams, api_users, api_stream, access_token = self.start(stream_type="rerun", broadcast_platform="rerun")
+    def test_disable_reruns_live(self, mock_log):
+        streams, api_users, api_stream, access_token = self.start()
+        self.assertTrue(api_users.called_once)
+        self.assertTrue(api_stream.called_once)
+        self.assertTrue(access_token.called_once)
+        self.assertTrue(streams)
+        self.assertNotIn(self.log_call, mock_log.info.call_args_list)
+
+    def test_disable_reruns_not_live(self, mock_log):
+        streams, api_users, api_stream, access_token = self.start(stream_type="rerun")
         self.assertTrue(api_users.called_once)
         self.assertTrue(api_stream.called_once)
         self.assertFalse(access_token.called)
@@ -249,20 +273,20 @@ class TestTwitchReruns(unittest.TestCase):
         self.assertDictEqual(streams, {})
         self.assertIn(self.log_call, mock_log.info.call_args_list)
 
-    def test_disable_reruns_live(self, mock_log):
-        streams, api_users, api_stream, access_token = self.start(stream_type="live", broadcast_platform="live")
+    def test_disable_reruns_broadcaster_software(self, mock_log):
+        streams, api_users, api_stream, access_token = self.start(broadcaster_software="watch_party_rerun")
         self.assertTrue(api_users.called_once)
         self.assertTrue(api_stream.called_once)
         self.assertTrue(access_token.called_once)
-        self.assertIsNone(streams)
-        self.assertNotIn(self.log_call, mock_log.info.call_args_list)
+        self.assertDictEqual(streams, {})
+        self.assertIn(self.log_call, mock_log.info.call_args_list)
 
     def test_disable_reruns_offline(self, mock_log):
         streams, api_users, api_stream, access_token = self.start(offline=True)
         self.assertTrue(api_users.called_once)
         self.assertTrue(api_stream.called_once)
         self.assertTrue(access_token.called_once)
-        self.assertIsNone(streams)
+        self.assertTrue(streams)
         self.assertNotIn(self.log_call, mock_log.info.call_args_list)
 
     def test_enable_reruns(self, mock_log):
@@ -270,5 +294,5 @@ class TestTwitchReruns(unittest.TestCase):
         self.assertFalse(api_users.called)
         self.assertFalse(api_stream.called)
         self.assertTrue(access_token.called_once)
-        self.assertIsNone(streams)
+        self.assertTrue(streams)
         self.assertNotIn(self.log_call, mock_log.info.call_args_list)
