@@ -7,7 +7,6 @@ import datetime
 import streamlink.cache
 
 from shutil import rmtree
-import time
 import unittest
 from tests.mock import patch
 
@@ -56,16 +55,16 @@ class TestCache(unittest.TestCase):
         self.cache.set("value", 10, expires=-20)
         self.assertEqual(None, self.cache.get("value"))
 
-    def test_expired_at(self):
+    def test_expired_at_before(self):
         self.cache.set("value", 10, expires_at=datetime.datetime.now() - datetime.timedelta(seconds=20))
         self.assertEqual(None, self.cache.get("value"))
 
-    def test_not_expired(self):
-        self.cache.set("value", 10, expires=20)
+    def test_expired_at_after(self):
+        self.cache.set("value", 10, expires_at=datetime.datetime.now() + datetime.timedelta(seconds=20))
         self.assertEqual(10, self.cache.get("value"))
 
-    def test_expired_at(self):
-        self.cache.set("value", 10, expires_at=datetime.datetime.now() + datetime.timedelta(seconds=20))
+    def test_not_expired(self):
+        self.cache.set("value", 10, expires=20)
         self.assertEqual(10, self.cache.get("value"))
 
     def test_create_directory(self):
@@ -104,7 +103,6 @@ class TestCache(unittest.TestCase):
         self.cache.set("test3", 3)
         self.cache.set("test4", 4)
 
-
         self.assertDictEqual(
             {"test3": 3, "test4": 4},
             self.cache.get_all())
@@ -113,7 +111,6 @@ class TestCache(unittest.TestCase):
         self.cache.set("test1", 1)
         self.cache.set("test2", 2, -1)
 
-
         self.assertDictEqual(
             {"test1": 1},
             self.cache.get_all())
